
#include "pcx_dbf.ch"

#command PrnGrSet( <r>, <c> )  => ?? cGrSet ; PrnGrMove( <r>, <c> )
#command PrnGrMove( <r>, <c> ) => ?? EVAL( bGrMove, <r>, <c> )
#command PrnGrReset( )         => ?? cGrReset
#command PrnGrLine( <r> )      => ?? EVAL( bGrLine, <r>)

STATIC cGrSet      // set the graphics mode
STATIC cGrReset    // resets it to normal
STATIC bGrMove     // move graphics cursor to specified position
STATIC bGrLine     // output given characters as pixels


//============================================================================
FUNCTION LoadPCX( cFile )
//----------------------------------------------------------------------------
// AUTHOR:     Denis A. Sarrazin (Tue 10-11-1994)
// PURPOSE:    to load the PCX into a special array
// PARAMETERS: cFile <-- the name of the PCX file to load
// RETURNS:    array (see list of #define in MINIPCX.CH and format below)
// NOTES:      Format of a .PCX header shown below:
//
//        Byte  Item            Size Description/Comments
//        ----  --------------- ---- -----------------------------------------
//        0     Manufacturer      1  Constant Flag  10 = ZSoft .PCX
//        1     Version           1  Version information:
//                                     0 = Version 2.5
//                                     2 = Version 2.8 w/palette information
//                                     3 = Version 2.8 w/o palette information
//                                     5 = Version 3.0
//        2     Encoding          1  1 = .PCX run length encoding
//        3     Bits per pixel    1  Number of bits/pixel per plane
//        4     Window            8  Picture Dimensions
//                                     (Xmin, Ymin) - (Xmax - Ymax)
//                                     in pixels, inclusive
//        12    HRes              2  Horizontal Resolution of creating device
//        14    VRes              2  Vertical Resolution of creating device
//        16    Colormap         48  Color palette setting, see text
//        64    Reserved          1
//        65    NPlanes           1  Number of color planes
//        66    Bytes per Line    2  Number of bytes per scan line per
//                                     color plane (always even for .PCX files)
//        68    Palette Info      2  How to interpret palette - 1 = color/BW,
//                                                              2 = grayscale
//        70    Filler           58  blank to fill out 128 byte header
//
//        IMPORTANT:
//          All sizes are measured in BYTES.
//          All variables of size 2 are integers.
//
//----------------------------------------------------------------------------
  LOCAL aPCX
  LOCAL cBuffer
  LOCAL hPCX
  LOCAL nRead
  LOCAL nFileLength

  IF !FILE(cFile)

    aPCX := {}

  ELSE

    aPCX  := ARRAY(PCX_LENGTH)
    hPCX  := FOPEN(cFile)
    nFileLength := FLEN(hPCX)

    cBuffer := SPACE(HEADER_SIZE)
    nRead := FREAD( hPCX, @cBuffer, HEADER_SIZE) // load PCX header

    aPCX[PCX_MANUFACTURER]   := ASC( SUBSTR(cBuffer,  1,  1) )
    aPCX[PCX_VERSION]        := ASC( SUBSTR(cBuffer,  2,  1) )
    aPCX[PCX_ENCODING]       := ASC( SUBSTR(cBuffer,  3,  1) )
    aPCX[PCX_BITS_PER_PIXEL] := ASC( SUBSTR(cBuffer,  4,  1) )
    aPCX[PCX_WINDOW]         := { BIN2I(SUBSTR(cBuffer,  5,  2)), ;
                                  BIN2I(SUBSTR(cBuffer,  7,  2)), ;
                                  BIN2I(SUBSTR(cBuffer,  9,  2)), ;
                                  BIN2I(SUBSTR(cBuffer, 11,  2)) }
    aPCX[PCX_HRES]           := BIN2I( SUBSTR(cBuffer, 13,  2) )
    aPCX[PCX_VRES]           := BIN2I( SUBSTR(cBuffer, 15,  2) )
    aPCX[PCX_COLORMAP]       := SUBSTR(cbuffer, 17, 48)
    aPCX[PCX_RESERVED]       := ASC( SUBSTR(cBuffer, 65,  1) )
    aPCX[PCX_NPLANES]        := ASC( SUBSTR(cBuffer, 66,  1) )
    aPCX[PCX_BYTES_PER_LINE] := BIN2I( SUBSTR(cBuffer, 67,  2) )
    aPCX[PCX_PALETTE_INFO]   := BIN2I( SUBSTR(cBuffer, 69,  2) )
    aPCX[PCX_FILLER]         := SUBSTR(cBuffer, 71, 58)

    cBuffer := SPACE(nFileLength - HEADER_SIZE)
    nRead := FREAD( hPCX, @cBuffer, nFileLength - HEADER_SIZE)
    IF (nRead != (nFileLength - HEADER_SIZE))
      ? "Warning: File too big!"
    ENDIF

    aPCX[PCX_PIXELS]         := cBuffer

    FCLOSE(hPCX)

  ENDIF

RETURN (aPCX)



//============================================================================
PROCEDURE PCXtoDBF( cPCXfile, cName )
//----------------------------------------------------------------------------
// AUTHOR:     Denis A. Sarrazin (Tue 10-11-1994)
// PURPOSE:    to append the given PCX file into the PCX DBF, named it cName
// PARAMETERS: cPCXDBF  <-- the name of the DBF to contain the PCX
//             cPCXfile <-- the name of the PCX file to load
//             cName    <-- the name to use for that PCX
//----------------------------------------------------------------------------
  LOCAL aPCX := LoadPCX(cPCXfile)

  PushDBF("PCX")

  LOCATE FOR TRIM(UPPER(cName)) == PCX->NAME

  IF FOUND()

    APPEND BLANK

    PCX->NAME       := cName
    PCX->MANUFACTUR := aPCX[PCX_MANUFACTURER]
    PCX->VERSION    := aPCX[PCX_VERSION]
    PCX->ENCODING   := aPCX[PCX_ENCODING]
    PCX->BITS_PIXEL := aPCX[PCX_BITS_PER_PIXEL]
    PCX->WIN_X1     := aPCX[PCX_WINDOW][1]
    PCX->WIN_Y1     := aPCX[PCX_WINDOW][2]
    PCX->WIN_X2     := aPCX[PCX_WINDOW][3]
    PCX->WIN_Y2     := aPCX[PCX_WINDOW][4]
    PCX->HRES       := aPCX[PCX_HRES]
    PCX->VRES       := aPCX[PCX_VRES]
    PCX->COLORMAP   := aPCX[PCX_COLORMAP]
    PCX->RESERVED   := aPCX[PCX_RESERVED]
    PCX->NPLANES    := aPCX[PCX_NPLANES]
    PCX->BYTES_LINE := aPCX[PCX_BYTES_PER_LINE]
    PCX->PALETTE    := aPCX[PCX_PALETTE_INFO]
    PCX->FILLER     := aPCX[PCX_FILLER]
    PCX->PICTURE    := aPCX[PCX_PIXELS]

    DBCOMMIT()

    UNLOCK

  ENDIF

  PopDBF() // PCX.DBF

RETURN


//============================================================================
FUNCTION DBFtoPCX( cName )
//----------------------------------------------------------------------------
// AUTHOR:     Denis A. Sarrazin (Tue 10-11-1994)
// PURPOSE:    to load the PCX from a database into a special array
// PARAMETERS: cName <-- the name of the PCX file to load
// RETURNS:    array (see list of #define in MINIPCX.CH and format below)
//----------------------------------------------------------------------------
  LOCAL aPCX := {}

  PushDBF("PCX")

  LOCATE FOR TRIM(UPPER(cName)) == PCX->NAME

  IF FOUND()

    aPCX  := ARRAY(PCX_LENGTH)

    aPCX[PCX_MANUFACTURER]   := PCX->MANUFACTUR
    aPCX[PCX_VERSION]        := PCX->VERSION
    aPCX[PCX_ENCODING]       := PCX->ENCODING
    aPCX[PCX_BITS_PER_PIXEL] := PCX->BITS_PIXEL
    aPCX[PCX_WINDOW]         := { PCX->WIN_X1, ;
                                  PCX->WIN_Y1, ;
                                  PCX->WIN_X2, ;
                                  PCX->WIN_Y2 }
    aPCX[PCX_HRES]           := PCX->HRES
    aPCX[PCX_VRES]           := PCX->VRES
    aPCX[PCX_COLORMAP]       := PCX->COLORMAP
    aPCX[PCX_RESERVED]       := PCX->RESERVED
    aPCX[PCX_NPLANES]        := PCX->NPLANES
    aPCX[PCX_BYTES_PER_LINE] := PCX->BYTES_LINE
    aPCX[PCX_PALETTE_INFO]   := PCX->PALETTE
    aPCX[PCX_FILLER]         := PCX->FILLER
    aPCX[PCX_PIXELS]         := PCX->PICTURE

  ENDIF

  PopDBF() // PCX.DBF

RETURN (aPCX)


//============================================================================
PROCEDURE PrintPCX( nRow, nCol, aPCX )
//----------------------------------------------------------------------------
// AUTHOR:     Denis A. Sarrazin (Tue 10-11-1994)
// PURPOSE:
// PARAMETERS:
// RETURNS:
// EXAMPLES:
//----------------------------------------------------------------------------
  LOCAL nNumCol     := aPCX[PCX_WINDOW][3] - aPCX[PCX_WINDOW][1] + 1
  LOCAL nTotalBytes := aPCX[PCX_NPLANES] * aPCX[PCX_BYTES_PER_LINE]
  LOCAL nStartCol   := nCol
  LOCAL cRasterData := ""
  LOCAL nCount, nNumBytes
  LOCAL cByte, nBytePos, nPixel
  LOCAL nDiff

  IF SetPrinter() // set the proper codes depending on type of printer

    PrnGrSet( nRow, nCol )

    nBytePos  := 1
    nNumBytes := 1
    WHILE (nBytePos < LEN(aPCX[PCX_PIXELS]))
      cByte := BYTE2BIN(ASC(SUBST(aPCX[PCX_PIXELS],nBytePos,1)))
      IF (LEFT(cByte,2) == "11")
        nCount := BIN2NUM("00"+SUBSTR(cByte,3))
        nBytePos++
        cByte := BYTE2BIN(ASC(SUBST(aPCX[PCX_PIXELS],nBytePos,1)))
      ELSE
        nCount := 1
      ENDIF
      WHILE (nCount > 0)
        // display all bits in byte (or parts thereof)
        IF ((nCol + 8) > (nNumCol + nStartCol - 1)) // chop off the excess
          nDiff := (nCol+8) - (nNumCol+nStartCol-1) - 1
          cByte := PADR(LEFT(cByte, 8 - nDiff), 8, "1") // clear any at end
        ENDIF
        cRasterData += CHR(XOR(BIN2NUM(cByte), 255))  // invert '1's and '0's
        nCol += 8
        IF (nNumBytes >= nTotalBytes)
          PrnGrMove( nRow, nStartCol )
          PrnGrLine( cRasterData )
          cRasterData := ""
          nNumBytes   := 0
          nCol        := nStartCol
          nRow++
        ENDIF
        nNumBytes++
        nCount--
      END
      nBytePos++
    END

    PrnGrReset()

  ENDIF

RETURN




//============================================================================
PROCEDURE PaintPCX( nRow, nCol, aPCX )
//----------------------------------------------------------------------------
// AUTHOR:     Denis A. Sarrazin (Tue 10-11-1994)
// PURPOSE:    to display a loaded PCX to the screen
// PARAMETERS: nRow <-- upper left corner on screen to start display of PCX
//             nCol <-- upper left corner on screen to start display of PCX
//             aPCX <-- array containing PCX info
// RETURNS:
// NOTES:      FOR each byte, X, read from the file
//               IF the top two bits of X are 1's
//                 then count = 6 lowest bits of X
//                 data = next byte following X
//               ELSE
//                 count = 1
//                 data = X
//----------------------------------------------------------------------------
  LOCAL nNumCol := aPCX[PCX_WINDOW][3] - aPCX[PCX_WINDOW][1] + 1
  LOCAL nTotalBytes := aPCX[PCX_NPLANES] * aPCX[PCX_BYTES_PER_LINE]
  LOCAL cByte, nBytePos, nPixel
  LOCAL nStartCol := nCol
  LOCAL nCount, nNumBytes
  LOCAL lGrMode := IF(GETMODE() == 18, .T., .F.)

  IF (!lGrMode)
    VIDEOMODE(18)
  ENDIF

  nBytePos  := 1
  nNumBytes := 1
  WHILE (nBytePos < LEN(aPCX[PCX_PIXELS]))
    cByte := BYTE2BIN(ASC(SUBST(aPCX[PCX_PIXELS],nBytePos,1)))
    IF (LEFT(cByte,2) == "11")
      nCount := BIN2NUM("00"+SUBSTR(cByte,3))
      nBytePos++
      cByte := BYTE2BIN(ASC(SUBST(aPCX[PCX_PIXELS],nBytePos,1)))
    ELSE
      nCount := 1
    ENDIF
    WHILE (nCount > 0)
      // display all bits in byte (or parts thereof)
      FOR nPixel := 1 TO 8
        IF (nCol < (nNumCol + nStartCol - 1))
          IF (SUBSTR(cByte, nPixel, 1) == "1")
            SETPIXEL(nRow, nCol,15)
          ELSE
            SETPIXEL(nRow, nCol, 0)
          ENDIF
        ENDIF
        nCol++
      NEXT n
      IF (nNumBytes >= nTotalBytes)
        nCol := nStartCol
        nNumBytes := 0
        nRow++
      ENDIF
      nNumBytes++
      nCount--
    END
    nBytePos++
  END

  IF (!lGrMode)
    VIDRESET()
  ENDIF

RETURN




//============================================================================
STATIC FUNCTION SetPrinter()
//----------------------------------------------------------------------------
// AUTHOR:     Denis A. Sarrazin (Fri 10-14-1994)
// PURPOSE:    to set up the appropriate strings based on the chosen printer
// PARAMETERS:
// RETURNS:    .T. if supported printer, .F. otherwise
// NOTE:       uses the default printer taken from RIGHTS.DBF
//----------------------------------------------------------------------------

  cGrSet := CHR(K_ESC)+"*t300R" + ;  // raster graphics at 300 dpi
            CHR(K_ESC)+"*r1A"        // start graphics at current position

  bGrMove := {|r,c| CHR(K_ESC)+"*p"+cNUM(r)+"Y" + ; // graphics vert cursor position
                    CHR(K_ESC)+"*p"+cNUM(c)+"X"}    // graphics horz cursor position

  cGrReset := CHR(K_ESC)+"*rB"  + ;  // end raster graphics
              CHR(K_ESC)+"&a0R" + ;  // move to text row 1
              CHR(K_ESC)+"&a0C"      // move to text column 1

  bGrLine := {|c| CHR(K_ESC)+"*b"+cNUM(LEN(c))+"W"+c } // send row of bits

RETURN (.T.)

