Program SfNodeWatch;
Uses CRT,DOS,BCShare;

(*        SpitFire Node Watch - FREEWARE by Richard Lemaster          *)
(*                                                                    *)
(*  This program will display a screen listing who is logged on to    *)
(*  each node of a SpitFire bbs. I'm including the source code so     *)
(*  Sysops will be able to design their display screen and maybe      *)
(*  programmers can learn something from the code. If you do modify   *)
(*  this code, please don't distribute it as SFNW. This program uses  *)
(*  Mike Woltz's BCShare file sharing unit which is available on      *)
(*  Buffalo Creek and many other BBS's.                               *)

Type
  NodesDat      = Record {Sfwhoson.dat}
                    UserNo : Integer;
                    Mode   : Char;
                    DBytes : Array[1..3] Of Byte;
                  End;

Var
  WorkDir:String[28];  {Spitfire work directory}
  UpdateSeconds:Word;  {Number of seconds between screen updates}
  Done:Boolean;        {Set to TRUE when Escape is pressed}
  NumberOfNodes:Byte;  {Number of nodes}
  Color1,              {Node: color}
  Color2,              {Border color}
  Color3:Byte;         {Text color}

Procedure GetWorkDir;
(* This procedure will read the SfNode.dat file to obtain the Work   *)
(* directory. It will work with SpitFire v3.4 & 3.5 because it uses  *)
(* the offset of the Work directory instead of reading the entire    *)
(* SfNode.dat file record. This unit might also work with versions   *)
(* earlier than 3.4 if the work dir field in SfNode.dat starts at    *)
(* position 19.                                                      *)

  Var
    A:File of Byte;  {SfNode.dat file}
    SLength:Byte;    {Length of Work dir string in SfNode.dat}
    TempByte:Byte;   {Used to read and build WorkDir}
    Temp:String;     {Spitfire dir obtained from SFPATH env. variable}
  Begin
    Temp:=GetEnv ('SFPATH');      {Check for SFPATH env. variable}
    If (Temp[Length(Temp)]<>'\') and (Temp<>'') then
      Temp:=Temp+'\';             {Add '\' if needed and SFPATH found.}
                                  {If SFPATH not found, checks current}
                                  {dir for SfNode.dat}
    Assign (A,Temp+'Sfnode.dat');
    SetFileMode (ReadMode);       {Don't let other programs write to file}
    {$I-} Reset (A); {$I+}        {Reset Sfnode.dat}
    If IoResult<>0 then           {Error trying to open Sfnode.dat}
      Begin
        Writeln (#10#13+Temp+'Sfnode.dat not found.');
        Halt (0);                 {Exit program}
      End;
    Seek (A,18);                  {Go to offset of WorkDir field in Sfnode}
    Read (A,SLength);             {Get length of WorkDir string in Sfnode}
    WorkDir:='';                  {Clear WorkDir string}
    For SLength:=1 to Ord (SLength) do {Ord SLength = Length of string}
      Begin
        Read (A,TempByte);        {Read next character in string}
        WorkDir:=WorkDir+ Chr(TempByte); {Add character to WorkDir string}
      End;
    Close (A);                    {Close SfNode.dat}
  End;

Procedure GetNumberOfNodes;
(* Reads SfSystem.dat to get the number of nodes. This procedure also    *)
(* goes to the offset of the NumberOfNodes field in SfSystem.dat instead *)
(* of reading the entire record from the file. Its a little faster and it*)
(* saves some memory.                                                    *)

  Var
    A:File of Byte;   {SfSystem.dat file}
  Begin
    Assign (A,WorkDir+'SfSystem.dat');
    SetFileMode (ReadMode);       {Don't let other programs write to file}
    {$I-} Reset (A); {$I+}        {Reset SfSystem.dat}
    If IoResult<>0 then           {Error trying to open SfSystem.dat}
      Begin
        Writeln (#10#13+WorkDir+'SfSystem.dat not found.');
        Halt;
      End;
    Seek (A,159);                 {Go to offset of NumberOfNodes field}
    Read (A,NumberOfNodes);       {Read NumberOfNodes from SfSystem.dat}
    Close (A);                    {Close file}
  End;

Function GetName (RecNum:Integer):String;
(* Gets name at position RecNum of SfUsers.ptr file. *)

  Type
    String30 = String[30];  {Users record type in SfUsers.ptr}
  Var
    A:File of String30;     {SfUsers.ptr file}
    Info:String30;          {Users record}
  Begin
    Assign (A,WorkDir+'Sfusers.ptr');
    SetFileMode (ReadMode);       {Don't let other programs write to file}
    {$I-} Reset (A); {$I+}
    If IoResult=0 then
      Begin
        Seek (A,RecNum);          {Go to callers record number in SfUsers.ptr}
        Read (A,Info);            {Read callers name from SfUsers.ptr}
        Close (A);                {Close file}
        If Length (Info)>26 then  {Check length of name so fits in display}
          Info[0]:=Chr(26);       {Set length to only 26 characters}
        GetName:=Info;            {Return callers name}
      End
    Else
      GetName:='';                {Couldn't open file so return blank name}
  End;

Procedure UpdateScreen;
(* Draw screen with node and caller information. *)

  Var
    I:Word;              {Counter}
    A:File of NodesDat;  {SfWhosOn.dat file}
    Info:NodesDat;       {SfWhosOn.dat file record}
  Begin
    ClrScr;
    Assign (A,WorkDir+'SfWhosOn.dat');
    SetFileMode (ReadDenyNone);   {Other programs can read or write to file}
    {$I-} Reset (A); {$I+}
    If IoResult<>0 then           {Couldn't open file}
      Begin
        Writeln (#10#13+'Error reading Sfwhoson.dat');
        Exit;
      End;
    TextColor (Color3);
    GotoXY (14,WhereY);           {Go to pos of title in display}
    Writeln ('SpitFire Node Watch - FREEWARE by Richard Lemaster');
    For I:=1 to NumberOfNodes do  {Read each node in Sfwhoson.dat}
      Begin
        ReadFile (FileRec (A).Handle,Info,6); {Read record from Sfwhoson.dat}
        TextColor (Color2);
        If I=1 then               {Draw top line of display if node 1}
          Writeln ('  Ŀ')
        Else                      {Draw middle line of display}
          Writeln ('  Ĵ');
        Write ('   ');
        TextColor (Color1);
        Write ('Node: ');
        TextColor (Color2);
        Write (I);                {Write node number}
        TextColor (Color3);
        GotoXY (14,WhereY);       {Go to pos of callers name in display}
        If Info.UserNo<0 then     {Check if caller online for node}
          Begin
            Write ('No Caller');
            TextColor (Color2);
            GotoXY (40,WhereY);   {Go to pos of middle  in display line}
            Write ('');
          End
        Else
          Begin
            Write (GetName (Info.UserNo));  {Get callers name}
            GotoXY (40,WhereY);             {Go to pos to display name}
            TextColor (Color2);
            Write ('');
            TextColor (Color3);
            GotoXY (42,WhereY);
            Case UpCase (Info.Mode) of     {Display what node is doing}
              'C': Write ('In Local Chat');
              'D': Write ('Operating Door');
              'N',
              'W': Write ('In Node Chat');
              'R': Write ('Using Message Base');
              'T': Write ('Transmitting File');
              'U': Write ('Unavailable For Chat');
            Else
              Write ('Available For Chat');
            end;
          End;
        TextColor (Color2);
        GotoXY (76, WhereY);      {Go to pos of right border}
        Writeln ('');
        If I=NumberOfNodes then   {If last node draw bottom of screen}
          Writeln ('  ')
      End;
    Close (A);                    {Close SfWhosOn.dat}
    TextColor (Color3);
    Write (#10#13+'  Display updated every ',UpdateSeconds,' seconds.  Press ESCAPE to quit');
  End;

Procedure NewDelay;
(* Checks if Escape pressed every second *)

  Var
    Choice:Char; {Key pressed to check for escape}
    I:Word;      {Counter}
  Begin
    For I:=1 to UpdateSeconds do
      Begin
        Delay (1000);              {1 Second delay}
        If KeyPressed then         {Check if key pressed}
          Begin
            Choice:=Readkey;       {Get key pressed}
            If Choice=#27 then     {Check if escape key}
              Begin
                I:=UpdateSeconds;  {Exit loop}
                Done:=True;        {Set Done to TRUE to exit program}
              End;
          End;
      End;
  End;

Procedure ChangeUpdateTime;
(* Changes update time if changed on command line. *)

  Var
    Error:Word;
    Temp:Word;
  Begin
    Val (ParamStr(1), Temp, Error); {Change command line parameter to a Word}
    If Error=0 then                 {Check that parameter was a number}
      UpdateSeconds:=Temp;
  End;

Procedure ProcessCommandLine;
(* Processes command line parameters to change update time and colors. *)

  Var
    Error:Word;
  Begin
    ChangeUpdateTime;             {Check for new update time on command line}
    If ParamCount>1 then          {Check command line for new colors}
      Val (ParamStr(2), Color1, Error);
    If ParamCount>2 then
      Val (ParamStr(3), Color2, Error);
    If ParamCount>3 then
      Val (ParamStr(4), Color3, Error);
  End;

Begin
  UpdateSeconds:=10;  {Default delay between screen updates}
  Done:=False;        {Set exit flag to FALSE}
  GetWorkDir;         {Get SF work dir}
  GetNumberOfNodes;   {Get number of nodes}
  Color1:=Cyan;       {Default colors}
  Color2:=LightCyan;
  Color3:=White;
  If ParamCount>0 then
    ProcessCommandLine;
  Repeat              {Keep updating screen until Done = TRUE}
    UpdateScreen;     {Draw screen}
    NewDelay;         {Delay between updates that checks for Escape key}
  Until Done;
  NormVideo;          {Restore original colors}
End.
